#include <stdio.h>
#include <string.h>

#define NAME_LEN 20
#define PHONE_LEN 11
#define MAX_ENTRIES 5

struct entry {
	char used;
	char name[NAME_LEN];
	char phone[PHONE_LEN];
};

struct entry entries[MAX_ENTRIES];


void phonebook_init(struct entry *entries, int n) {
  int i;

  for (i=0; i<n; i++) { 
    entries[i].used = 0; 
  }

}

int find0(const struct entry *entries, int n, 
          const char *name){
  int i; 

  for (i=0; (i<n) && (!entries[i].used || strcmp(entries[i].name,name)); i++);

  return(i);
}

int phonebook_find(const struct entry *entries, int n,
                   const char *name, char *phone) {
  int pos;

  pos = find0(entries, n, name);

  if (pos == n) { 
    return(0); 
  }
  else { 
    strcpy(phone, entries[pos].phone); 
    return(1); 
  }

}

int phonebook_rmv(struct entry *entries, int n,
                  const char name[]) {
  int pos;

  pos = find0(entries, n, name);

  if (pos < n) {
    entries[pos].used = 0;
    return(1); 
  }
  else {
    return(0);
  }

}

int phonebook_add(struct entry *entries, int n,
                  const char *name, const char *phone) {
  int pos;

  pos = find0(entries, n, name);

  if (pos < n) {
    strcpy(entries[pos].phone, phone); 
    return(-1);  /* existing entry replaced */
  }
  
  for (pos=0; (pos<n) && (entries[pos].used); pos++);
  if (pos == n) { 
    return(0);  /* no free space */
  }		
  
  strcpy(entries[pos].name, name);
  strcpy(entries[pos].phone, phone);
  entries[pos].used = 1; 
  return(1); /* new entry added */
}

void phonebook_print(const struct entry *entries, int n) {
	int i;
	
	printf("Contents BEGIN\n");
	for (i=0; i<n; i++) {
		if (entries[i].used) {
			printf("Name = %s, Phone = %s\n", entries[i].name, entries[i].phone);
		}
	}
	printf("Contents END\n");

	}


int main(int argc, char *argv[]) {
	struct entry entries[MAX_ENTRIES];
	int res;
	char name[NAME_LEN], phone[PHONE_LEN];
	
	phonebook_init(entries, MAX_ENTRIES);
	
	while (1) {
		while (1) {
			printf("Enter name and number to add: ");
			res = scanf("%19s %10s", name, phone);
			if (res != 2) {
				printf("Bad input!\n");
				return(1);
			}
			res = phonebook_add(entries, MAX_ENTRIES, name, phone);
			if (res == 1)  
				printf("Entry added\n");
			else if (res == -1)
				printf("Entry replaced\n");
			else {
				printf("No space");
				break;
			}
		}
	
		phonebook_print(entries, MAX_ENTRIES);
	
		while (1) {
			printf("Enter name to search: ");
			res = scanf("%19s", name);
			if (res != 1) {
				printf("Bad input!\n");
				return(1);
			}
			res = phonebook_find(entries, MAX_ENTRIES, name, phone);
			if (res == 1)
				printf("Found %s\n", phone);
			else {
				printf("Not found\n");
				break;
			}
		}
	
		while (1) {
			printf("Enter name to remove: ");
			res = scanf("%19s", name);
			if (res != 1) {
				printf("Bad input!\n");
				return(1);
			}
			res = phonebook_rmv(entries, MAX_ENTRIES, name);
			if (res == 1) {
				printf("Removed\n");
				phonebook_print(entries, MAX_ENTRIES);
			}
			else {
				printf("Not found\n");
				break;
			}
		}
		/* and all over again */
	}
	
	return(0);
}
